///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: AR-DS-StreamInfo.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which encapsulates all of the details
//   of an audio stream that is being played by the DirectSound based
//   audio renderer.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include <mmsystem.h>
#include <dsound.h>


// The length of the buffer for a stream (in milliseconds)
const int BUFFER_LENGTH = 300;

// How often these buffers will be updated (in milliseconds)
const int BUFFER_UPDATE_TIME = 100;


class AudioSourceProxy;


class StreamInfo
{
public:
	LPDIRECTSOUND m_pDS;			// DirectSound object which controls the sound card
	WAVEFORMATEX m_wf;				// Format of the audio being played
	DSBUFFERDESC m_bufDesc;			// DirectSound buffer descriptor
	LPDIRECTSOUNDBUFFER m_pDSBuf;	// DirectSound streaming buffer object
	DWORD m_startTime;				// When to start playing this buffer
	AudioSourceProxy* m_pSrc;		// Pointer to the node to get data from
	DWORD m_nextWrite;				// Buffer offset of where to write next data
	BOOL m_playing;					// TRUE if the stream is playing.
	FILE* m_rawOutput;				// Only for debugging

	// Constructor, set default values
	StreamInfo(LPDIRECTSOUND pDS, AudioSourceProxy* pSrc, DWORD startTime);

	// Destructor, shut down the stream.
	~StreamInfo();

	// Create a streaming buffer.  Return DS_OK if created OK.
	int Create(DWORD nSamplesPerSec, DWORD nBitsPerSample, WORD nNumChannels);

	// Calculate the maximum size of sound buffer data which can be written
	// to the given stream.  i.e. how much free space there is in buffer.
	DWORD GetMaxWriteSize();

	// Write the given data to the stream buffer.
	void WriteData(LPBYTE pData, DWORD nNumBytes);

	// Write the given number of bytes worth of silence into the buffer.
	void WriteSilence(DWORD nNumBytes);

	// Start the stream playing
	void Play();

	// Return TRUE if the format of this stream is known, otherwise
	// return FALSE.
	BOOL IsFormatKnown() const { return (m_wf.nChannels != 0); }

	// Return TRUE if the stream is already playing, FALSE otherwise.
	BOOL IsPlaying() const { return m_playing; }

}; /* class StreamInfo */


// ===============================================================================
