///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Appearance2DProxy.h
//
// Authors: Antonio Arena, CSELT, arena@kim.cselt.it
//			Franco Casalino, CSELT, Franco.Casalino@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the Appearance2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef APPEARANCE2D_PROXY_H
#define APPEARANCE2D_PROXY_H


#include "Traversable.h"


class Appearance2DProxy : public Traversable
{
public:
	Appearance2DProxy(MediaObject* pMO) : Traversable(pMO) {}

	// Return a pointer to the Appearance2D node that this node is the proxy for
	Appearance2D* GetAppearance2D() const
	{ return (Appearance2D*)GetObject(); }

	// Return a pointer to the Material2D object belonging to the
	// Appearance2D node that this node is the proxy for.
	Material2D* GetMaterial2D() const
	{ return (Material2D*)(GetAppearance2D()->material); }

	// Return a pointer to the Properties2D object belonging to the
	// Appearance2D node that this node is the proxy for.
	Properties2D* GetProperties2D() const
	{ return (Properties2D*)(GetAppearance2D()->properties); }

	// Functions inherited from the base class
	void PreRender(Effects const& /*effects*/) {}
	void Print(FILE* outfile, int depth) const;
	BOOL IsFinished() const { return TRUE; }
};


#endif // APPEARANCE2D_PROXY_H
