///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  AudioSourceProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the AudioSource node.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUDIO_SOURCE_PROXY_H
#define AUDIO_SOURCE_PROXY_H


#include "Nodes-CSELT.h"
#include "Traversable.h"


class AudioSourceProxy : public Traversable
{
public:

	AudioSourceProxy(MediaObject* pMO);
	~AudioSourceProxy();

	AudioSource* GetAudioSource() const { return (AudioSource*)GetObject(); }
	// Return a pointer to the AudioSource node that this node is the proxy for

	MediaStream* GetStream() const { return GetAudioSource()->GetStream(); }
	// Return a pointer to the stream that this node reads from.

	void GetData(DWORD nBytesRequired, LPBYTE& rpData, DWORD& rnBytesGiven);
	// This function is used by the audio renderer to get more data
	// to play.  The audio render will ask for a given number of bytes
	// (which will be a multiple of the number of bytes required for a
	// single sample), and the audio source node will return a pointer
	// to some data and the number of bytes of data pointed to.
	//
	// The number of bytes returned can be less than that asked for.
	// If fewer bytes were returned to the audio renderer than expected,
	// then the audio renderer will ask for more.  If zero bytes are
	// returned, then the audio renderer will assume that no data can be
	// given, and will insert silence.

	void DataUsed(LPBYTE pData, DWORD nBytesGiven);
	// This function is used by the audio renderer to indicate that it
	// has finished with some data given to it by GetData().  The
	// parameters given to this function will be the same as those
	// returned by GetData().

	// Functions inherited from the base class
	void PreRender(Effects const& effects);
	void Print(FILE* outfile, int depth) const;
	float GetDrawingOrder() const { return (float)0; }
	BOOL IsFinished() const { return m_isFinished; }


private:
	BOOL m_registeredWithRenderer;
	BOOL m_skippedHeader;
	DWORD m_partialTime;
	DWORD m_partialLen;
	DWORD m_partialStoreAllocatedSize;
	unsigned char* m_partialStore;
	BOOL m_needsReleasing;
	BOOL m_isFinished;
	long m_startUpDelay;
	BOOL m_firstTimeAtStartTime;
	BOOL m_firstTimeAtStopTime;
	int m_timesStarted;
};


#endif // AUDIO_SOURCE_PROXY_H


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
