///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Background2DProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Background2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Background2DProxy.h"


// =====================================================================

//
// PresenterCSELT::InitBackground2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitBackground2D(Background2D* pObj)
{
	pObj->SetPresenterData(new Background2DProxy(pObj));
}


// =====================================================================

//
// Background2DProxy::Background2DProxy
//
// Class constructor.
//

Background2DProxy::Background2DProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTime = TRUE;
} /* Background2DProxy::Background2DProxy */


// =====================================================================

//
// Background2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Background2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sBackground2D {\n", IndentPrefix(depth));
	fprintf(outfile, "%sobjectDescriptorID %d\n",
			IndentPrefix(depth + 1),
			GetBackground2D()->GetObjectDescriptorID());
	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Background2DProxy::Print */


// =====================================================================

//
// Background2DProxy::PreRender
//
// Call the PreRender() method from the base class, and then adjust the
// bounding rectangle which has just been calculated, so that the
// top-left corner of the bounding rectangle is at the top-left corner
// of the screen.
//

void Background2DProxy::PreRender(Effects const& effects)
{
	// The first time that this is called, the stream needs to be
	// started.
	if (m_firstTime)
	{
		m_firstTime = FALSE;
		GetBackground2D()->OpenStream();
	}

	DrawableImage::PreRender(effects);
	m_boundingRect.left = (int)GetVisualRenderer()->GetScreenWidth() / -2;
	m_boundingRect.top  = (int)GetVisualRenderer()->GetScreenHeight() / 2;

	UpdateBoundingRect();
} /* Background2DProxy::PreRender */


// =====================================================================
