///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.							 //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  G723Dec.cpp
//
// Description: G723 Decoder class implementation
//   .
// Created: September, 1997.
//
// Modified: Patrizio Bergallo, CSELT, patrizio.bergallo@cselt.it
//
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include <assert.h>
#include <limits.h>
#include "G723Dec.h"
#include "Caps.h"
#include "Stream.h"
#include "trace.h"

#include "g723.h"

static FILE* fIn  = NULL;
static FILE* fOut = NULL;

/*-----------------------------------------------------------------
Constructor
-----------------------------------------------------------------*/
G723Decoder :: G723Decoder (class Executive *pExecutive) : Decoder (pExecutive)
{
	m_dwTime = 0;
	m_firstDecode = TRUE;

//	fIn  = fopen ("decoderInput.raw", "wb");
//	fOut = fopen ("decoderOutput.pcm", "wb");
}

/*-----------------------------------------------------------------
Destructor
-----------------------------------------------------------------*/
G723Decoder :: ~G723Decoder()
{
	if (fIn != NULL)
		fclose(fIn);
	if (fOut != NULL)
		fclose(fOut);
}

/*-----------------------------------------------------------------
Set the decoder output format and initialize the decoder
-----------------------------------------------------------------*/
FORMAT G723Decoder :: SetFormat (Capabilities *pCaps)
{
	int i;

	for (i = 0; i < pCaps -> GetFormatCapabilityLength (); i++)
		if (pCaps -> GetFormatCapability (i) == FORMAT_PCM)
		{	
			G723InitDec();
			return FORMAT_PCM;
		}
	return FORMAT_UNKNOWN;
}		

/*-----------------------------------------------------------------
Decompress a unit.
-----------------------------------------------------------------*/
BOOL G723Decoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	int nError;

	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}
	if (fIn != NULL)
		fwrite(pInput,nInputLength,1,fIn);

	// The first time that this function is called, write the format
	// of the audio (which is always the same).
	if (m_firstDecode)
	{
		m_firstDecode = FALSE;

		DecoderAudioFormat* pFmt = (DecoderAudioFormat*)pOutput;
		pFmt->samplesPerSecond = 8000;
		pFmt->bitsPerSample	   = 16;
		pFmt->numChannels	   = 1;

		nError = G723Dec ((char *)(pOutput + sizeof(DecoderAudioFormat)), (char *)pInput);
		rnOutputLength = GetOptimalOutputSize();

		if (fOut != NULL)
			fwrite(pOutput + sizeof(DecoderAudioFormat),480,1,fOut);
	}
	else
	{
		nError = G723Dec ((char *)pOutput, (char *)pInput);
		rnOutputLength = 480;

		if (fOut != NULL)
			fwrite(pOutput,480,1,fOut);
	}
	assert (nError != 0);

	return TRUE;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void G723Decoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;

	while (
		(pInData =
			(LPBYTE) GetInputStream() -> FetchNext (nInSize, dwTime, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		if (m_bActive)
		{	nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) GetOutputStream() -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
			{	GetInputStream() -> Terminate ();
				return;
			}
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) GetOutputStream() -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
				{	GetInputStream() -> Terminate ();
					return;
				}
			}
			GetOutputStream() -> Dispatch (nOutSize, m_dwTime);
			m_dwTime += 30;
		}
		GetInputStream() -> Release (TRUE);
	}
	GetOutputStream() -> Close ();
	m_streamClosed = TRUE;
}
