///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  GeometryNode.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which contains details specific to
//   geometry nodes.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef GEOMETRY_NODE_H
#define GEOMETRY_NODE_H


#include "DrawableNode.h"
#include "Nodes-CSELT.h"

class GeometryNode : public DrawableNode
{
	//
	// Public functions
	//
public:

	// Class constructor - set the pointer to the media object that
	// this object is attached to, and also the pointer to the visual
	// renderer.
	GeometryNode(MediaObject* pMO) : DrawableNode(pMO) {}

	// Set and get the appearance for this geometry node.
	void SetAppearance(Appearance2D* pNode) { m_pAppearance = pNode; }
	Appearance2D* GetAppearance() const { return m_pAppearance; }

	// Set and get the pointer to the parent Shape2DProxy node
	void SetShape2DProxy(class Shape2DProxy* pShape2DProxy)
	{ m_pShape2DProxy = pShape2DProxy; }
	Shape2DProxy* GetShape2DProxy() const { return m_pShape2DProxy; }

	// Returns TRUE if this geometry node needs to be drawn (because it
	// has changed since it was last drawn).  This will be true if the
	// node has been updated (via BIFS or ROUTES).
	virtual BOOL NeedsDrawing() const;

	// Assume that the Geometry node will be transparent (which is
	// always the case for Circle and Text).  Other classes can override
	// this if required.
	virtual BOOL IsTransparent() const { return TRUE; }

	// Functions to get the values from the Appearance2D, Material2D and
	// Properties2D nodes, which use the default values if these node are
	// missing.
	Color3f GetDiffuseColor() const;
	float GetTransparency() const;
	Bool GetFilled() const;
	Bool GetShadow() const;
	float GetShadPosX() const;
	float GetShadPosY() const;
	int GetWidth() const;

	// Functions inherited from base classes
	virtual void FindTouchSensor(EventInfo const* pEvent);
	BOOL IsFinished() const { return TRUE; }


	//
	// Private implementation details
	//
private:
	Appearance2D* m_pAppearance;
	class Shape2DProxy* m_pShape2DProxy;
};


#endif // GEOMETRY_NODE_H
