///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  GroupNodeProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which encapsulates all of the features
//   of a node which are common to nodes which have children.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef GROUP_NODE_PROXY_H
#define GROUP_NODE_PROXY_H


#include "Traversable.h"


class GroupNodeProxy : public Traversable
{
public:
	GroupNodeProxy(MediaObject* pMO) : Traversable(pMO) {}

	// Return a pointer to the GroupNode node that is the base class of
	// the node that this is the proxy for
	GroupNode* GetGroupNode() const { return (GroupNode*)GetObject(); }

	// Return a pointer to the Nth child of this group
	Traversable* GetChild(int n)
	{ return (Traversable*)GetGroupNode()->GetChildren()[n]->GetPresenterData(); }
	Traversable* GetChild(int n) const
	{ return (Traversable*)GetGroupNode()->GetChildren()[n]->GetPresenterData(); }

	// Functions inherited from the base class
	virtual void UpdateTimeSensor();
	void FindTouchSensor(EventInfo const* pEvent);
	BOOL IsFinished() const;


	// Methods to be called only by classes deriving from this class
protected:
	void PreRenderChildren(Effects const& effects);
	void PrintChildren(FILE* outfile, int depth) const;
};


#endif // GROUP_NODE_PROXY_H
