/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       H263GDec.cpp
AUTHOR:     Karl Anders ygard (Telenor R&D)
OVERVIEW:   Implementation of H263Decoder
-------------------------------------------------------------------------*/
#include "defs.h"
#include <assert.h>
#include "H263Dec-unix.h"
#include "../H263Decoder/bitio.h"

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
FORMAT H263Decoder :: SetFormat (Capabilities *)
{
	return FORMAT_YUV;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int H263Decoder :: GetOptimalOutputSize ()
{
	return 176 * 144 * 3 / 2 + (int)sizeof(DecoderImage);
}


BOOL H263Decoder :: Decode (LPBYTE pInput, int nInputLength,
							LPBYTE pOutput, int &rnOutputLength)
{
	// Decode the next frame
	bitiob inbuf((unsigned char*) pInput, 0, nInputLength);
	unsigned char *frame[3];
	m_decoder.Decode(inbuf, frame);

	// If there is not enough buffer space, return FALSE
	int frameSize = m_decoder.horizontal_size * m_decoder.vertical_size;
	if (rnOutputLength < frameSize * 3 / 2 + (int)sizeof(DecoderImage))
	{
		rnOutputLength = frameSize * 3 / 2 + (int)sizeof(DecoderImage);
		return FALSE;
	}

	// Store the dimensions of the frame in the output buffer
	DecoderImage* pImgDetails = (DecoderImage*)pOutput;
	pImgDetails->width   = m_decoder.horizontal_size;
	pImgDetails->height  = m_decoder.vertical_size;
	pImgDetails->pitch   = m_decoder.horizontal_size;
	pImgDetails->offsetX = 0;
	pImgDetails->offsetY = 0;

	// Copy the output of the decoder into the output buffer
	pOutput += sizeof(DecoderImage);
	memcpy(pOutput, frame[0], frameSize);
	memcpy(pOutput + frameSize, frame[1], frameSize / 4);
	memcpy(pOutput + frameSize * 5 / 4, frame[2], frameSize / 4);

	rnOutputLength = frameSize * 3 / 2 + (int)sizeof(DecoderImage);
	return TRUE;
}


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
