///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Image2DProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Image2D node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Image2DProxy.h"


// =====================================================================

//
// PresenterCSELT::InitImage2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitImage2D(Image2D* pObj)
{
	pObj->SetPresenterData(new Image2DProxy(pObj));
}


// =====================================================================

//
// Image2DProxy::Image2DProxy
//
// Class constructor.
//

Image2DProxy::Image2DProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTime = TRUE;
} /* Image2DProxy::Image2DProxy */


// =====================================================================

//
// Image2DProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void Image2DProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sImage2D {\n", IndentPrefix(depth));
	fprintf(outfile, "%sobjectDescriptorID %d\n",
			IndentPrefix(depth + 1),
			GetImage2D()->GetObjectDescriptorID());
	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* Image2DProxy::Print */


// =====================================================================

//
// Image2DProxy::PreRender
//
// Call the PreRender() and UpdateBoundingRect() methods from the base
// class.
//

void Image2DProxy::PreRender(Effects const& effects)
{
	// The first time that this is called, the stream needs to be
	// started.
	if (m_firstTime)
	{
		m_firstTime = FALSE;
		GetImage2D()->OpenStream();
	}

	DrawableImage::PreRender(effects);
	UpdateBoundingRect();
} /* Image2DProxy::PreRender */


// =====================================================================
