/****************************************************************************/
/*                                                            				*/
/* Copyright (c) 1997 ISO/IEC All Rights Reserved                			*/
/*                                                            				*/
/* This software module was developed by									*/
/*																			*/
/* CSELT  Centro Studi e Laboratori Telecomunicazioni                       */
/*                                                            				*/
/* in the course of development of the MPEG-4 standard. This software       */
/* module is an implementation of a part of one or more MPEG-4 tools as     */
/* specified by the MPEG-4 standard.             							*/
/*                                                            				*/
/* The copyright of this software belongs to ISO/IEC. ISO/IEC gives use     */
/* of the MPEG-4 standard free license to use this  software module or      */
/* modifications thereof for hardware or software products claiming         */
/* conformance to the MPEG-4 standard        								*/
/*                                                            				*/
/* Those intending to use this software module in hardware or software      */
/* products are advised that use may infringe existing  patents. The        */
/* original developers of this software module and their companies, the     */
/* subsequent editors and their companies, and ISO/IEC have no liability    */
/* and ISO/IEC have no liability for use of this software module or         */
/* modification thereof in an implementation.                               */
/*                                                            				*/
/****************************************************************************/

/****************************************************************************
 *
 * File:        Decoder.cpp
 *
 * Author:      Franco Casalino, Antonio Arena
 * Created:     28.05.97
 *
 * Modified:    Matthew Leditschke, 1/7/97
 *
 * Description: JPEG Decoder class definition
 * 
 * Comments: Work derived from the Independent JPEG Group's software
 *
 ****************************************************************************/

#include "defs.h"
#include <assert.h>
#include "JPEGDec.h"
#include "jpegmemdec.h"


BOOL JPEGDecoder::Decode(LPBYTE pInput,int nInputLength,LPBYTE pOutput,int &rnOutputLength)
{
	// The in-memory decoder does not allow for the storage of the 
	// dimensions of the image.
	rnOutputLength -= sizeof(DecoderImage);

	// Decode the image
	WORD width, height;
	BOOL result = JpegDecodeInMemory(pInput, nInputLength,
									 pOutput + sizeof(DecoderImage),
									 &rnOutputLength, &width, &height);

	// If the width and height are zero, then an error has occurred
	// decoding the image
	if ((width == 0) || (height == 0))
		assert(0);

	// Store the dimensions of the image in the output buffer
	DecoderImage* pImgDetails = (DecoderImage*)pOutput;
	pImgDetails->width   = width;
	pImgDetails->height  = height;
	pImgDetails->pitch   = width * 3;
	pImgDetails->offsetX = 0;
	pImgDetails->offsetY = 0;

	rnOutputLength += sizeof(DecoderImage);
	return result;
}


FORMAT JPEGDecoder :: SetFormat(Capabilities * pPresentationCapabilities)
{
	FORMAT format;
	for (int i = 0; i < pPresentationCapabilities->GetFormatCapabilityLength(); i++) {
		format = pPresentationCapabilities->GetFormatCapability(i);
		if (format == FORMAT_RGB)
			return FORMAT_RGB;
	}
	return FORMAT_UNKNOWN;
}

int JPEGDecoder :: GetOptimalInputSize()
		{ return (15500); }
int JPEGDecoder :: GetOptimalOutputSize()
		{ return ((352 * 288 * 3) + 30); }
int JPEGDecoder :: GetRecommendedInputBufferSize()
		{ return 3 * GetOptimalInputSize(); }
int JPEGDecoder :: GetRecommendedOutputBufferSize()
		{ return 3 * GetOptimalOutputSize(); }



