// MPEG4PlayerDoc.cpp : implementation of the CMPEG4PlayerDoc class
//

#include "defs.h"
#include <assert.h>
#include "trace.h"
#include "Clock.h"
#include "MPEG4Player.h"
#include "MainFrm.h"
#include "MPEG4PlayerDoc.h"
#include "EnterNetworkAddress.h"
#include "Presenter-CSELT.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc

IMPLEMENT_DYNCREATE(CMPEG4PlayerDoc, CDocument)

BEGIN_MESSAGE_MAP(CMPEG4PlayerDoc, CDocument)
	//{{AFX_MSG_MAP(CMPEG4PlayerDoc)
	ON_COMMAND(ID_FILE_RESTART, OnFileRestart)
	ON_UPDATE_COMMAND_UI(ID_FILE_RESTART, OnUpdateFileRestart)
	ON_COMMAND(ID_FILE_OPEN_NETWORK, OnFileOpenNetwork)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc construction/destruction

CMPEG4PlayerDoc::CMPEG4PlayerDoc()
{
	m_fileOpened = FALSE;
	m_fileName.Empty();
}

CMPEG4PlayerDoc::~CMPEG4PlayerDoc()
{
}

BOOL CMPEG4PlayerDoc::OnNewDocument()
{
//	if (!CDocument::OnNewDocument())
//		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc serialization

void CMPEG4PlayerDoc::Serialize(CArchive& /*ar*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc diagnostics

#ifdef _DEBUG
void CMPEG4PlayerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMPEG4PlayerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerDoc::DocumentNeedsOpening
//
// This returns TRUE if a file needs to be opened when the executive is
// started.  This only happens when a file is opened in response to being
// double clicked.
//

BOOL CMPEG4PlayerDoc::DocumentNeedsOpening()
{
	return ((!m_fileName.IsEmpty()) && (!m_fileOpened));
} /* CMPEG4PlayerDoc::DocumentNeedsOpening */


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerDoc::OpenDocument
//
// Open the file whose name is given by the m_fileName data member of
// the class.  Because Executive::Start() is an asynchronous function,
// we can only assume that the file has been successfully opened when
// Application::OnConnect() is called.
//

BOOL CMPEG4PlayerDoc::OpenDocument() 
{
	// Close the current file - if one is open
	if (m_fileOpened)
		CloseDocument();

	// If the file name stored in m_fileName ends with "<dmif>", then
	// it is a network address.
	if (m_fileName.Right(6) == "<dmif>")
	{
		// Extract the address from the name of the file:
		//  - remove leading path and trailing "<dmif>"
		int start = m_fileName.ReverseFind('\\');
		assert(start != -1);
		int end = m_fileName.Find("<dmif>");
		assert(end != -1);
		assert(end > start);
		CString addressName = m_fileName.Mid(start + 1, end - start - 1);

		// Start the executive running, giving it the name of the address.
		TRACE_PRINT("Starting the executive, given an address.\n");
		GetApp()->GetExecutive()->Start(REMOTE_SERVICE, addressName);
	}

	else
	{
		// Start the executive running, giving it the name of the file.
		TRACE_PRINT("Starting the executive, given a file.\n");
		GetApp()->GetExecutive()->Start(FILE_SERVICE, m_fileName);
	}

	return TRUE;
} /* CMPEG4PlayerDoc::OpenDocument */


/////////////////////////////////////////////////////////////////////////////

BOOL CMPEG4PlayerDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	// Remember the name of the file
	m_fileName = lpszPathName;

#ifdef SIMULATE_BIFS_DECODER
	extern char* globalFileName;

	if (globalFileName != NULL)
		free(globalFileName);

	// Store the name of the file in a global variable, used when
	// simulating the BIFS decoder, so that it knows which scene
	// to create.
	// Get the base file name to decide which type of scene to create
	const char* tmpPtr = strrchr(lpszPathName, '\\');
	ASSERT(tmpPtr != NULL);
	tmpPtr++;
	globalFileName = _strdup(tmpPtr);
#endif

	// If the executive has not yet been created (as is the case if
	// this application started in response to a file being double
	// clicked, then delay opening the document).
	if (GetApp()->GetExecutive() == NULL)
		return TRUE;
	else
		// Open the document with this name
		return OpenDocument();
} /* CMPEG4PlayerDoc::OnOpenDocument */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnFileOpenNetwork() 
{
	CEnterNetworkAddress dialog;
	int result = dialog.DoModal();
	
	if (result == IDOK)
	{
		const char* pAddress = dialog.GetNetworkAddress();

		// Append <dmif> to the end of the name of the network address,
		// so that OpenDocument() can tell the difference
		char* tempName = new char [strlen(pAddress) + 7];
		strcpy(tempName, pAddress);
		strcat(tempName, "<dmif>");

		// Give this (psuedo) filename to the application framework
		// to open, which means that OpenDocument() will end up with it.
		GetApp()->OpenDocumentFile(tempName);

		delete tempName;
	}

} /* CMPEG4PlayerDoc::OnFileOpenNetwork */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnFileRestart() 
{
	// Only proceed if there is a previous file to restart
	if (!m_fileName.IsEmpty())
	{
		// Close the current file
		CloseDocument();

		// Open the previous file, and start it playing
		OpenDocument();
	}
} /* CMPEG4PlayerDoc::OnFileRestart */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::CloseDocument() 
{
	// Only proceed a file is actually open
	if (m_fileOpened)
	{
		// Stop the executive running this session
		GetApp()->GetExecutive()->Stop();
		m_fileOpened = FALSE;

		// Wait for the application to be notified that the file
		// has been closed
		GetApp()->m_smpFileOpen.P();
		GetApp()->m_smpFileOpen.V();
	}
} /* CMPEG4PlayerDoc::CloseDocument */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnCloseDocument() 
{
	TRACE_PRINT("Entering CMPEG4PlayerDoc::OnCloseDocument()\n");
	CloseDocument();	
	CDocument::OnCloseDocument();
	TRACE_PRINT("Leaving CMPEG4PlayerDoc::OnCloseDocument()\n");
} /* CMPEG4PlayerDoc::OnCloseDocument */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnUpdateFileRestart(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_fileName.IsEmpty());
}


/////////////////////////////////////////////////////////////////////////////
