///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Nodes-CSELT.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The file includes the header files of all of the nodes that have
//   been incorporated into the CSELT 2D player.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef NODES_CSELT_H
#define NODES_CSELT_H


#include "StreamCo.h"
#include "NodeFld.h"
#include "bifs.h"
#include "Executiv.h"
#include "Presenter-CSELT.h"
#include "Root.h"


#define eventIn		 public:
#define exposedField public:
#define eventOut	 public:
#define field		 public:


// ======================================================================

class Appearance2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	 exposedField SFNode<class Material2D> material;
	 exposedField SFTextureNode texture;
	 exposedField SFNode<class TextureTransform> textureTransform;
	 exposedField SFNode<class Properties2D> properties;

public:
	Appearance2D() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitAppearance2D(this); }
#endif
};


// ======================================================================

class AudioSource : public StreamConsumer
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<float> pitch;
	exposedField SF<Time> startTime;
	exposedField SF<Time> stopTime;

public:
	AudioSource()
	{
		pitch = 1;
		startTime = 0;
		stopTime = 0;
	}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitAudioSource(this); }
#endif
};


// ======================================================================

class Background2D : public StreamConsumer
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn  SF<Bool> set_bind;
	eventOut SF<Bool> isBound;

public:
	Background2D() {}

	void Onset_bind() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitBackground2D(this); }
#endif
};


// ======================================================================

class Circle : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<float> radius;

public:
	Circle() :
		radius(1)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitCircle(this); }
#endif
};


// ======================================================================

class Color : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField MF<Color3f> color;

public:
	Color() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitColor(this); }
#endif
};


// ======================================================================

class ColorInterpolator : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn		 SF<float> set_fraction;
	exposedField MF<float> key;
	exposedField MF<Color3f> keyValue;
	eventOut	 SF<Color3f> value_changed;

public:
	ColorInterpolator() {}

	void Onset_fraction();

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitColorInterpolator(this); }
#endif
};


// ======================================================================

class Coordinate2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField MF<Vec2f> point;

public:
	Coordinate2D() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitCoordinate2D(this); }
#endif
};


// ======================================================================

class Curve2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SFNode<Coordinate2D> points;
	exposedField SF<int> fineness;

public:
	Curve2D() :
		fineness(0)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitCurve2D(this); }
#endif
};


// ======================================================================

class FontStyle2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SFString type;
	exposedField SF<int> style;
	exposedField SF<int> size;
	exposedField SF<int> hSpacing;
	exposedField SF<int> vSpacing;
	exposedField SFString language;

public:
	FontStyle2D() :
		type(""),
		style(0),
		size(-1),
		hSpacing(100),
		vSpacing(100),
		language("")
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitFontStyle2D(this); }
#endif
};


// ======================================================================

class Group2D : public Root
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn		 MFNodeType<SF2DNode> addChildren;
	eventIn		 MFNodeType<SF2DNode> removeChildren;
	exposedField MFNodeType<SF2DNode> children;
	field		 SF<Vec2f> bboxCenter;
	field		 SF<Vec2f> bboxSize;

public:
	Group2D() : 
		bboxCenter (Vec2f(0,0)),
		bboxSize (Vec2f(-1,-1))
	{}

    void OnaddChildren() {}
    void OnremoveChildren() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitGroup2D(this); }
#endif
};


// ======================================================================

class Image2D : public StreamConsumer
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

public:
	Image2D() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitImage2D(this); }
#endif
};


// ======================================================================

class IndexedLineSet2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn MF<int> set_colorIndex;
	eventIn MF<int> set_coordIndex;
	exposedField SFNode<Color> color;
	exposedField SFNode<Coordinate2D> coord;
	field MF<int> colorIndex;
	field SF<Bool> colorPerVertex;
	field MF<int> coordIndex;

public:
	IndexedLineSet2D() :
		colorPerVertex(TRUE)
	{}

	void Onset_colorIndex() {}
	void Onset_coordIndex() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitIndexedLineSet2D(this); }
#endif
};


// ======================================================================

class Layout : public GroupNode
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField MFNodeType<SF2DNode> children;
	exposedField SF<int> mode;
	exposedField SF<Bool> wrap;
	exposedField SF<float> width;
	exposedField SF<float> height;
	exposedField SF<float> lineSpacing;
	eventIn		 SFGenericNode addChildren;
	eventIn		 SFGenericNode removeChildren;

public:
	Layout() : 
		mode(0),
		wrap(FALSE),
		width(-1),
		height(-1),
		lineSpacing(1)
	{}

	void OnaddChildren();
	void OnremoveChildren();

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitLayout(this); }
#endif
};


// ======================================================================

class Material2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

public:
	exposedField SF<Color3f> diffuseColor;       
	exposedField SF<float> transparency;   

public:
	Material2D() :
		diffuseColor(Color3f(0.8, 0.8, 0.8)),
		transparency(0)	
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitMaterial2D(this); }
#endif
};


// ======================================================================

class Movie2D : public StreamConsumer
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<Bool> loop;
	exposedField SF<float> speed;
	exposedField SF<Time> startTime;
	exposedField SF<Time> stopTime;
	eventOut	 SF<float> duration_changed;
	eventOut	 SF<Bool> isActive;

public:
	Movie2D() :
		loop(FALSE),
		speed(1),
		startTime(0),
		stopTime(0),
		isActive(FALSE)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitMovie2D(this); }
#endif
};


// ======================================================================

class PointSet2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SFNode<Color> color;
	exposedField SFNode<Coordinate2D> coord;

public:
	PointSet2D() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitPointSet2D(this); }
#endif
};


// ======================================================================

class Position2DInterpolator : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn		 SF<float> set_fraction;
	exposedField MF<float> key;
	exposedField MF<Vec2f> keyValue;
	eventOut	 SF<Vec2f> value_changed;

public:
	Position2DInterpolator() {}

	void Onset_fraction();

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitPosition2DInterpolator(this); }
#endif
};


// ======================================================================

class Properties2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<Bool> filled;
	exposedField SF<Bool> shadow;
	exposedField SF<Vec2f> shadPos;
	exposedField MF<int> dotted;
	exposedField SF<int> width;

public:
	Properties2D() :
	  filled(FALSE),
	  shadow(FALSE),
	  shadPos(Vec2f(5,5)),
	  width(1)
	{
#ifndef BIFSENC
		dotted += 2;
		dotted += 2;
#endif
	}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitProperties2D(this); }
#endif
};


// ======================================================================

class Rectangle : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<Vec2f> size;

public:
	Rectangle() : size(Vec2f(2,2))	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitRectangle(this); }
#endif
};


// ======================================================================

class ScalarInterpolator : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn		 SF<float> set_fraction;
	exposedField MF<float> key;
	exposedField MF<float> keyValue;
	eventOut	 SF<float> value_changed;

public:
	ScalarInterpolator() {}

	void Onset_fraction();

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitScalarInterpolator(this); }
#endif
};


// ======================================================================

class Scroll : public GroupNode
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn		 SFGenericNode addChildren;
	eventIn		 SFGenericNode removeChildren;
	exposedField MFNodeType<SF2DNode> children;
	exposedField SF<Bool> smoothScroll;
	exposedField SF<Bool> loop;
	exposedField SF<float> scrollXRate;
	exposedField SF<float> scrollYRate;
	exposedField SF<float> width;
	exposedField SF<float> height;

public:
	Scroll() : 
		smoothScroll(FALSE),
		loop(FALSE),
		scrollXRate(0),
		scrollYRate(0),
		width(-1),
		height(-1)
	{}

	void OnaddChildren() {}
	void OnremoveChildren() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitScroll(this); }
#endif
};


// ======================================================================

class Shape2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SFNode<class Appearance2D> appearance;
	exposedField SFGeometry2DNode geometry;

public:
	Shape2D() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitShape2D(this); }
#endif
};


// ======================================================================

class Sound2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<Vec2f> location;
	exposedField SF<float> intensity;
	exposedField SFAudioNode sound;
	exposedField SF<Bool> spatialize;

public:
	Sound2D() : 
		location(Vec2f(0, 0)),
		intensity(1),
		spatialize(TRUE)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitSound2D(this); }
#endif
};


// ======================================================================

class Switch : public GroupNode
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField MFGenericNode choice;
	exposedField SF<int> whichChoice;

public:
	Switch() : 
		whichChoice(-1)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitSwitch(this); }
#endif
};


// ======================================================================

class Text2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SFString data;
	exposedField SF<float> length;
	exposedField SFNode<class FontStyle2D> fontStyle;
	exposedField SF<Bool> ucs_2;

public:
	Text2D() :
		data(""),
		length(-1),
		ucs_2(FALSE)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitText2D(this); }
#endif
};


// ======================================================================

class TextureTransform : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

public:
	TextureTransform() {}
};


// ======================================================================

class TimeSensor : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<Time> cycleInterval;
	exposedField SF<Bool> enabled;
	exposedField SF<Bool> loop;
	exposedField SF<Time> startTime;
	exposedField SF<Time> stopTime;
	eventOut	 SF<Time> cycleTime;
	eventOut	 SF<float> fraction_changed;
	eventOut	 SF<Bool> isActive;
	eventOut	 SF<Time> time;

public:
	TimeSensor() :
		cycleInterval(1),
		enabled(TRUE),
		loop(FALSE),
		startTime(0),
		stopTime(0)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitTimeSensor(this); }
#endif
};


// ======================================================================

class TouchSensor2D : public MediaObject
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField SF<Bool> enabled;
	eventOut	 SF<Vec2f> hitPoint_changed;
	eventOut	 SF<Vec2f> hitTexCoord_changed;
	eventOut	 SF<Bool> isActive;
	eventOut	 SF<Bool> isOver;
	eventOut	 SF<Time> touchTime;

public:
	TouchSensor2D() :
		enabled(TRUE)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitTouchSensor2D(this); }
#endif
};


// ======================================================================

class Transform2D : public GroupNode
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	eventIn		 MFNodeType<SF2DNode> addChildren;
	eventIn		 MFNodeType<SF2DNode> removeChildren;
	exposedField SF<Vec2f> center;
	exposedField MFNodeType<SF2DNode> children;
	exposedField SF<float> rotationAngle;
	exposedField SF<Vec2f> scale;
	exposedField SF<float> scaleOrientation;
	exposedField SF<float> drawingOrder;
	exposedField SF<Vec2f> translation;
	field		 SF<Vec2f> bboxSize;
	field		 SF<Vec2f> bboxCenter;

public:
	Transform2D() : 
		center (Vec2f(0,0)),
		rotationAngle (0),
		scale (Vec2f(1,1)),
		scaleOrientation (0),
		drawingOrder (0),
		translation (Vec2f(0,0))
	{}

	void OnaddChildren() {}
	void OnremoveChildren() {}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitTransform2D(this); }
#endif
};

// ======================================================================

class XLayout : public GroupNode
{
	friend BIFSDecoder;
	BIFS_DECLARE_NODE

	exposedField MFNodeType<SF2DNode> children;
	exposedField SF<float> width;
	exposedField SF<float> height;
	field		 MF<float> groups;
	field		 MF<float> constraint;

public:
	XLayout() :
		width(-1),
		height(-1)
	{}

#ifndef BIFSENC
	void RegisterObject()
	{ ((PresenterCSELT*)GetExecutive()->GetPresenter())->InitXLayout(this); }
#endif
};


// ======================================================================

#include "CrNodes.h"


#endif // NODES_CSELT_H
