///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: AudioSourceProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a simple AudioSourceProxy node used for testing
//   purposes.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUDIO_SOURCE_PROXY_H
#define AUDIO_SOURCE_PROXY_H


class AudioRenderer;


class AudioSourceProxy
{
	//
	// Public functions
	//
public:

	AudioSourceProxy();

	~AudioSourceProxy();


	// Open the given file, with the associated audio format.  Return
	// TRUE if opened OK, otherwise return FALSE.
	int Init(char* fileName, DWORD samplesPerSec, DWORD bitsPerSample,
			 WORD numChannels, AudioRenderer* pAudRend);

	// This function is used by the audio renderer to get more data
	// to play.  The audio render will ask for a given number of bytes
	// (which will be a multiple of the number of bytes required for a
	// single sample), and the audio source node will return a pointer
	// to some data and the number of bytes of data pointed to.
	//
	// The number of bytes returned can be less than that asked for.
	// If fewer bytes were returned to the audio renderer than expected,
	// then the audio renderer will ask for more.  If zero bytes are
	// returned, then the audio renderer will assume that no data can be
	// given, and will insert silence.
	void GetData(DWORD nBytesRequired, LPBYTE& rpData, DWORD& rnBytesGiven);

	// This function is used by the audio renderer to indicate that it
	// has finished with some data given to it by GetData().  The
	// parameters given to this function will be the same as those
	// returned by GetData().
	void DataUsed(LPBYTE pData, DWORD nBytesGiven);


	//
	// Private implementation details
	//
private:

	FILE* m_file;
	DWORD m_bufferSize;
	char* m_buffer;
	AudioRenderer* m_pAudRend;
	DWORD m_samplesPerSec;
	DWORD m_bitsPerSample;
	WORD m_numChannels;
	long m_bytesLeftInFile;
};


#endif // AUDIO_SOURCE_PROXY_H
