///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  TimeSensorProxy.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to handle the TimeSensor node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef TIME_SENSOR_PROXY_H
#define TIME_SENSOR_PROXY_H


#include "Traversable.h"


class TimeSensorProxy : public Traversable
{
public:
	TimeSensorProxy(MediaObject* pMO);

	// Return a pointer to the TimeSensor node that this node is the proxy for
	TimeSensor* GetTimeSensor() const
	{ return (TimeSensor*)GetObject(); }

	// Functions inherited from the base classes
	void UpdateTimeSensor();
	void PreRender(Effects const&) {}
	void Print(FILE* outfile, int depth) const;
	BOOL IsFinished() const;


	// Private implementation details
private:
	BOOL m_isActive;
	BOOL m_prevEnabled;
	BOOL m_prevActive;
	BOOL m_endOfTimeReached;
	double m_cycleStartTime;

	// Local copies of the TimeSensor values which cannot be changed
	// once the node is active
	double m_startTime;
	double m_cycleInterval;
	BOOL m_enabled;
	BOOL m_loop;
};


#endif // TIME_SENSOR_PROXY_H
