///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  Traversable.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of methods of the class which encapsulates all
//   of the features of a nodes used by the visual renderer.
//   This file contains the implementation of methods common to all nodes
//   which are traversable.
//
// Created: August, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "Group.h"
#include "Traversable.h"


// ========================================================================

//
// Traversable::Traversable
//
// Class constructor.  Set the pointer to the media object that
// this object is attached to, and get pointers to the presenter
// and the visual and audio renderers, which can be used by
// derived classes.
//

Traversable::Traversable(MediaObject* pMO) : m_pMediaObject(pMO)
{
	m_pPresenter      = (PresenterCSELT*)pMO->GetExecutive()->GetPresenter();
	m_pVisualRenderer = m_pPresenter->GetVisualRenderer();
	m_pAudioRenderer  = m_pPresenter->GetAudioRenderer();
} /* Traversable::Traversable */


// ========================================================================

//
// Traversable::IndentPrefix
//
// Return a string containing the correct number of spaces for the given
// level of indentation.  This function is called by the Print() method
// of each node when printing out the scene tree.
//

const char* Traversable::IndentPrefix(int depth) const
{
	static char spaces[200];

	memset(spaces, ' ', depth * 4);
	spaces[depth * 4] = 0;

	return spaces;
} /* Traversable::IndentPrefix */


// ========================================================================

//
// Traversable::BoolToStr
//
// Return a string which displays the value of the given Bool.
//

const char* Traversable::BoolToStr(Bool b) const
{
	if (b)
		return "TRUE";
	else
		return "FALSE";
} /* Traversable::BoolToStr */


// ========================================================================

//
// Traversable::FindTouchSensor
//
// This is the default behaviour for a leaf node, which involves calling
// FindTouchSensor() of its enclosing grouping node.
//

void Traversable::FindTouchSensor(EventInfo const* pEvent)
{
	// Go through all of the parents
	int whichParent = 0;
	GroupNode* pParent = GetObject()->GetParent(whichParent);
	while (pParent != NULL)
	{
		((Traversable*)pParent->GetPresenterData())->FindTouchSensor(pEvent);
		whichParent++;
		pParent = GetObject()->GetParent(whichParent);
	}
} /* Traversable::FindTouchSensor */


// ========================================================================
