///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: VisRend-CSELT-DD.h
//
// Author: Matthew Leditschke <CSELT, Matthew.Leditschke@cselt.it>
//
// Description:
//   The definition of a class which performs visual rendering using
//   DirectDraw under Windows 95/NT.
//
// Created: August, 1997
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef VIS_REND_CSELT_DD_H
#define VIS_REND_CSELT_DD_H


#include "VisRend.h"
#include "DirectDraw.h"


class VisualRendererDD : public VisualRenderer
{
	//
	// Public functions
	//
public:

	// Class constructor - given a pointer to the presenter and a handle
	// to the window to display in.
	VisualRendererDD(Presenter* pPresenter, HWND hWnd);

	// Class destructor.
	virtual ~VisualRendererDD();

	// Methods inherited from the VisualRenderer base class.
	virtual void Init();
	virtual void Terminate();
	virtual void Paint(void* pInfo = NULL);
	virtual void Clear(BRect const* pUpdateRect = NULL);
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void SetScreenSize(unsigned int /*newX*/,
							   unsigned int /*newY*/) {}
	virtual void DisplayRGBAImage(SrcRGBAImage const& srcImage);
	virtual void DisplayYUV420AImage(SrcYUVAImage const& srcImage);
	virtual void DisplayCircle(BRect const& pClipBRect, PointF const* lpPoints,
							   Color3f color, int width, 
							   BOOL filled);
    virtual void DisplayCurve(BRect const& pClipBRect, 
							  PointF const* lpPoints,
							  Color3f color, 
							  int width, 
							  int length);
    virtual void DisplayLines(BRect const& pClipBRect, 
							  int const& startPos1, int const& startPos2,
							  int const& endPos1, int const& endPos2,
							  Color3f const& pixelColor, 
							  int width);
	virtual void DisplayPolygon(BRect const& clipBRect,
								PointF const* lpPoints, int numPoints,
								Color3f const& pixelColor,
								int width, BOOL filled);
	virtual void DisplayRectangle(BRect const& clipBRect,
								  BRect const& drawBRect,
								  Color3f const& pixelColor,
								  int width, BOOL filled,
								  float transparency);
	virtual void DisplayPoint(int xPos, int yPos, Color3f const* pPixelColor,
							  float transparency);
	virtual void* SetFont(const char* fontName, int fontSize, BOOL bold,
						  BOOL italics, BOOL underline);
	virtual void UnsetFont(void* pFont);
	virtual void GetTextSize(void* pFont, const char* string, int length,
							 unsigned int& width, unsigned int& height);
	virtual void DisplayText(void* pFont, const char* pString, int length,
							 int x, int y, Color3f const& color,
							 BRect const* pClipBRect);

	virtual void GetScreenSize(unsigned int& currX,
							   unsigned int& currY) const
	{ currX = m_outputWidth; currY = m_outputHeight; }

	virtual unsigned int GetScreenWidth() const
	{ return m_outputWidth; }

	virtual unsigned int GetScreenHeight() const
	{ return m_outputHeight; }


	//
	// Private implementation details
	//
private:

	// Remember the handle to the window in which the drawing takes place
	HWND m_hWnd;

	// The DirectDraw object which accesses the DirectDraw libraries
	DirectDraw* m_pDirectDraw;

	// The dimensions of the output window
	unsigned int m_outputWidth;
	unsigned int m_outputHeight;

};


#endif // VIS_REND_CSELT_DD_H
