///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: VisRend-CSELT-X11.h
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The definition of a class which performs visual rendering using
//   the X11 libraries on unix.
//
// Created: September, 1997
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef VIS_REND_CSELT_X11_H
#define VIS_REND_CSELT_X11_H


#include "VisRend.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>


// The different types of RGB displays that are supported
enum RGBpixelType { RGB32bpp, BGR32bpp };


class VisualRendererX11 : public VisualRenderer
{
	//
	// Public functions
	//
public:

	// Class constructor - given a pointer to the presenter and the name
	// of the display to open.
	VisualRendererX11(Presenter* pPresenter, const char* displayName);

	// Class destructor - calls terminate if it hasn't already been called
	~VisualRendererX11();

	// Methods inherited from the VisualRenderer base class.
	virtual void Init();
	virtual void Terminate();
	virtual void Paint(void* pInfo = NULL);
	virtual void Clear(BRect const* pUpdateRect = NULL);
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void SetScreenSize(unsigned int newX, unsigned int newY);
	virtual void DisplayRGBAImage(SrcRGBAImage const& srcImage);
	virtual void DisplayYUV420AImage(SrcYUVAImage const& srcImage);

	virtual void GetScreenSize(unsigned int& currX, unsigned int& currY) const
	{ currX = m_outputWidth; currY = m_outputHeight; }

	virtual unsigned int GetScreenWidth() const
	{ return m_outputWidth; }

	virtual unsigned int GetScreenHeight() const
	{ return m_outputHeight; }

	// Return the current display
	Display* GetDisplay() const { return m_display; }


	//
	// Private implementation details
	//
private:

	// Remember the name of the display being used
	char const* m_displayName;

	// The display being used
	Display* m_display;

	// The window being used
	Window m_window;

	// The Visual being used
	Visual *m_visual;

	// The RGB pixel format of the visual
	RGBpixelType m_RGBpixelFormat;

	// The graphics context being used
	GC m_gc;

	// The color depth of the window
	int m_depth;

	// Remember a pointer to the presenter that this renderer is part of
	Presenter* m_pPresenter;

	// The dimensions of the output window
	unsigned int m_outputWidth;
	unsigned int m_outputHeight;

	// The XImage structure which is drawn into
	XImage* m_ximage;

	// The memory associated with this XImage
	unsigned int* m_imageData;


	//
	// Private functions
	//
	Visual* FindFullColorVisual();
	Window CreateFullColorWindow (int x, int y, int w, int h);

};


#endif // VIS_REND_CSELT_X11_H


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
