///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File:  XLayoutProxy.cpp
//
// Author: Matthew Leditschke, CSELT, Matthew.Leditschke@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the XLayout node.
//
// Created: October, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "XLayoutProxy.h"


// =====================================================================

//
// PresenterCSELT::InitXLayout
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

void PresenterCSELT::InitXLayout(XLayout* pObj)
{
	pObj->SetPresenterData(new XLayoutProxy(pObj));
}


// =====================================================================

//
// XLayoutProxy::PreRender
//
// Apply the PreRender method to all of the children of this node, and
// then, using their bounding boxes, change their position according to
// the layout parameters.
//

void XLayoutProxy::PreRender(Effects const& effects)
{
	PreRenderChildren(effects);

	int childLength = GetXLayout()->children.GetLength();
	int mode;
	for (int i = 0; i <	GetXLayout()->constraint.GetLength(); i++)
	{
		if ((mode = GetXLayout()->constraint[i]) != -1)
		{
			i++;
			while(GetXLayout()->constraint[i] != -1)
			{
				if (GetXLayout()->constraint[i] <= childLength)
				{
					Apply(mode, GetChild(GetXLayout()->constraint[i]));
					i++;
				}
				else
				{
					Apply(mode, GetXLayout()->constraint[i]);
					i++;
				}
			}
		}
		else break;
	}
} /* XLayoutProxy::PreRender */


// =====================================================================

//
// XLayoutProxy::Apply
//
// Apply the transformations to the Layout's children according to the
// mode type.
//

void XLayoutProxy::Apply(int /*mode*/, Traversable* /*node*/)
{
} /* XLayoutProxy::Apply */


// =====================================================================

//
// XLayoutProxy::Apply
//
// Apply the transformations to the Layout's children according to the
// mode type.
//

void XLayoutProxy::Apply(int mode, int group)
{
	int i = group;
	while (GetXLayout()->groups[i] != -1)
	{
		Apply(mode, GetChild(GetXLayout()->constraint[GetXLayout()->groups[i]]));
		i ++;
	}
} /* XLayoutProxy::Apply */


// =====================================================================

//
// XLayoutProxy::Print
//
// Print out the fields and corresponding values for the media object
// derived node that this object is a proxy for, indented according to
// the given depth.
//

void XLayoutProxy::Print(FILE* outfile, int depth) const
{
	fprintf(outfile, "%sXLayout {\n", IndentPrefix(depth));

	// Print out the children of this node
	PrintChildren(outfile, depth + 1);

	// Print out the width & height fields of this node
	const char* indentStr = IndentPrefix(depth + 1);
	fprintf(outfile, "%swidth %.2f\n", indentStr, GetXLayout()->width);
	fprintf(outfile, "%sheight %.2f\n", indentStr, GetXLayout()->height);

	// Print out the integers in the groups field
	fprintf(outfile, "%sgroups [", indentStr);
	for (int i = 0; i < GetXLayout()->groups.GetLength(); i++)
		fprintf(outfile, " %d", (int)GetXLayout()->groups[i]);
	fprintf(outfile, " ]\n");

	// Print out the integers in the constraint field
	fprintf(outfile, "%sconstraint [", indentStr);
	for (i = 0; i < GetXLayout()->constraint.GetLength(); i++)
		fprintf(outfile, " %d", (int)GetXLayout()->constraint[i]);
	fprintf(outfile, " ]\n");

	fprintf(outfile, "%s}\n", IndentPrefix(depth));
} /* XLayoutProxy::Print */


// =====================================================================
