#if !defined(_BifsDec_h)	// Sentry, use file only if it's not already included.
#define _BifsDec_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       BifsDec.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Declarations of BIFSDecoder.
-------------------------------------------------------------------------*/

class Executive;
class MediaStream;
class MediaObject;
class Root;

class BIFSDecoder
{
	Executive *m_pExecutive;
	MediaStream *m_pBIFS;
	Root *m_pScene;
	int m_nLastBit,	m_nSize, m_nLastByte;
	LPBYTE m_pData;

private:
	void ParseSceneReplaceCommand();
	void ParseReplacementCommand ();
	void ParseDeletionCommand ();
	void ParseInsertionCommand ();
	void ParseUpdateFrame ();
	void ParseRoute (Root *pRoot, int nID = -1);
	void ParseRoutesDescription (Root *pRoot);
	void ParseSessionStreamAssociation ();
	void ParseObjectDesc();


public:
	BIFSDecoder (Executive *pExecutive, MediaStream *pBIFS);
	~BIFSDecoder ();
	MediaObject *BuildScene ();
	void UpdateScene ();
	int ParseInt (int nBits);
	float ParseFloat ();
	double ParseDouble ();

	Executive *GetExecutive () {return m_pExecutive;}

#ifdef _DEBUG
private:
	int m_nIndent;
	BOOL m_bNewLine;
	FILE *m_fTrace;

public:
	void Tracef (LPCSTR szFormat, ...);
	void TraceIndent (int nDirection);
	void TraceNewLine ();
#endif
};

enum BifsExcept
{	BIFX_PARSING_ERROR,
	BIFX_END_OF_STREAM,
	BIFX_UNSUPPORTED_LONG_STRING,
	BIFX_NODEID_NOT_FOUND
};
	
/*-------------------------------------------------------------------------
Define a class for exceptions during node parsing
-------------------------------------------------------------------------*/
class BifsException
{
public:
	enum BifsExcept m_uErrorCode;
	
	BifsException (enum BifsExcept uErrorCode = BIFX_PARSING_ERROR) :
		m_uErrorCode (uErrorCode)
	{}
};

typedef enum tagBifsUpdateCode {
	SESSIONSTREAMASSOCIATION = 99,
	UPDATEFRAME = 1
} BIFSUPDATECODE;


#ifdef _DEBUG
#define TRACEF(format, text) pBifs -> Tracef (format, text)
#define TRACEF2(format, x, y) pBifs -> Tracef (format, x, y)
#define TRACEF3(format, x, y, z) pBifs -> Tracef (format, x, y, z)
#define TRACEF4(format, x, y, z, w) pBifs -> Tracef (format, x, y, z, w)
#define TRACEINDENT(direction) pBifs -> TraceIndent (direction)
#define TRACENEWLINE pBifs -> TraceNewLine ()
#else
#define TRACEF(format, text)
#define TRACEF2(format, x, y)
#define TRACEF3(format, x, y, z)
#define TRACEF4(format, x, y, z, w)
#define TRACEINDENT(direction)
#define TRACENEWLINE
#endif	// _DEBUG

#endif                                      // _BifsDec_h sentry.
