#if !defined(_channel_h)	// Sentry, use file only if it's not already included.
#define _channel_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Adaptation Layer
FILE:       Channel.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Declarations of Channel and DataChannel
-------------------------------------------------------------------------*/
#include "Im1defs.h"
#include "ESConfig.h"

class MediaStream;
class Service;

/*-----------------------------------------------------------------
Channel is a base class for DataChannel and FlexDemux
-----------------------------------------------------------------*/
class Channel
{
public:
	virtual void OnPacketReceived (LPBYTE pData, int nPacketLength) = 0;
	virtual ~Channel() {}
};

class DataChannel : public Channel
{
	
	MediaStream *m_pStream;
	Service *m_pService;
	ESCONFIGPARAMS m_configParams;
	DWORD m_dwCTS, m_dwPreCTS, m_dwTimeOffset;
	int m_nAULength;
	int m_nExpectedAULength;
	LPBYTE m_pBuffer, m_pData;
	int m_nESid, m_nTag;
	int m_nLastByte, m_nLastBit, m_nCount;
	BOOL m_bChannelReady;

private:
	int GetByte ();
	int GetBits (int nBits);
	void ResetByteCount () {m_nCount = 0;}
	int GetByteCount () {return m_nCount;}

public:
	DataChannel (Service *pService, int nESid) :
		m_pService (pService),
		m_nESid (nESid),
		m_nTag (-1),
		m_bChannelReady (FALSE)
	{};
	~DataChannel ();
	int GetESid () {return m_nESid;}
	void Config (
		MediaStream *pStream,
		ESCONFIGPARAMS *pConfigParams,
		int nExpectedAULength = 256
	);
	virtual void OnPacketReceived (LPBYTE pData, int nPacketLength);
	void ChannelReady ();
	void OnChannelOpen (int nTag);
};

#endif                                      // _channel_h sentry.
