/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ BifsEncoder
FILE:       Common.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Functions which are common to the decoder and the encoder
-------------------------------------------------------------------------*/
#include "defs.h"
#include <assert.h>
#include "MediaObj.h"
#include "NodeFld.h"
#include "bifs.h"

/*-----------------------------------------------------------------
Find address of a field in the node given its BifsTable entry
-----------------------------------------------------------------*/
NodeField *MediaObject :: GetField (const BifsFieldTable *pEntry)
{	
	return (NodeField *) (((char *) this) + pEntry -> m_offset);
}

/*-----------------------------------------------------------------
Find address of a field in the node, given its index
-----------------------------------------------------------------*/
NodeField *MediaObject :: GetField (int nIndex)
{
	return GetField (GetFieldTable () + nIndex);
}

/*-----------------------------------------------------------------
SFGenericNode include a static member which is an array of factory
objects which used to instantiate nodes
-----------------------------------------------------------------*/
BifsFactory *SFGenericNode :: m_bifsFactory [MPEG4_TOTAL_NUMBER_OF_NODES];

/*-----------------------------------------------------------------
Every SF...Node class has a table of nodes of that type. SFGenericNode
includes all the nodes, therefore GetNodeTableSize() returns the
total amount of nodes defined in MPEG4.
-----------------------------------------------------------------*/
int SFGenericNode :: GetNodeTableSize ()
{
	return MPEG4_TOTAL_NUMBER_OF_NODES;
}

/*-----------------------------------------------------------------
Stores a pointer to a factory object in the factory array. This
function is called from factory object (BifsFactory) constructor.
A factory object for each node is instantiated through the use of the
BIFS_IMPLEMENT_NODE_START macro.
-----------------------------------------------------------------*/
void SFGenericNode :: StoreFactory (int nCode, BifsFactory *pFactory)
{
	assert (
		nCode >= 0 && nCode < MPEG4_TOTAL_NUMBER_OF_NODES &&
		m_bifsFactory [nCode] == NULL
	);
	m_bifsFactory [nCode] = pFactory;
}

/*-----------------------------------------------------------------
Use the factory arrays to instantiate a proper node. This is a new
implementation of SFGenericNode :: CreateNode() replacing the old
implementation which used a primitive long switch command.
-----------------------------------------------------------------*/
MediaObject *SFGenericNode :: CreateNode (int nCode)
{
	if (nCode < 0)
		return NULL;
	assert (nCode < MPEG4_TOTAL_NUMBER_OF_NODES);
	if (m_bifsFactory [nCode] != NULL)
		return m_bifsFactory [nCode] -> CreateNode ();
	return NULL;
}