/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       Decoder.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of Decoder - base class for all decoders.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Decoder.h"
#include "Stream.h"

/*-----------------------------------------------------------------
Decoder constructor
-----------------------------------------------------------------*/
Decoder :: Decoder (Executive *pExecutive)
{
	m_pExecutive = pExecutive;
	m_pInStream = NULL;
}

/*-----------------------------------------------------------------
Close() makes sure the Decoder thread has finished (which
happens when the input stream returns END_OF_STREAM).
-----------------------------------------------------------------*/
void Decoder :: Close ()
{
	m_bActive = FALSE;
	m_smpActive.P ();
	m_smpActive.V ();

	/*-----------------------------------------------------------------
	Delete the input stream (the output stream will be deleted by the
	MediaObject
	-----------------------------------------------------------------*/
	delete m_pInStream;
}

/*-----------------------------------------------------------------
Start() stores the stream object pointers, and creates the Decoder
thread.
-----------------------------------------------------------------*/
void Decoder :: Start (
	MediaStream *pCodedStream,
	MediaStream *pPresentationStream
)
{
	m_pInStream = pCodedStream;
	m_pOutStream = pPresentationStream;
	m_bActive = TRUE;
	m_smpActive.P ();
	if (ZThread<Decoder> (DecoderThread, this).Begin () == -1)
	{	m_smpActive.V ();
    	throw;
	}
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void Decoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;

	while (
		(pInData =
			(LPBYTE) m_pInStream -> FetchNext (nInSize, dwTime, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		if (m_bActive)
		{	nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) m_pOutStream -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
			{	m_pInStream -> Terminate ();
				return;
			}
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) m_pOutStream -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
				{	m_pInStream -> Terminate ();
					return;
				}
			}
			m_pOutStream -> Dispatch (nOutSize, dwTime);
		}
		m_pInStream -> Release (TRUE);
	}
	m_pOutStream -> Close ();
}

/*-----------------------------------------------------------------
The Decoder thread function calls the virtual Run() function and 
release the flag semaphore when finishes.
-----------------------------------------------------------------*/
void Decoder :: DecoderThread ()
{
	Run ();
	m_smpActive.V ();
}