/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       G723GDec.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of G723Decoder (simulation - just copies
			the units)
-------------------------------------------------------------------------*/
#include "Defs.h"
#include <assert.h>
#include <limits.h>
#include "G723Dec.h"
#include "Caps.h"
#include "Stream.h"

//static FILE *fOut;

/*-----------------------------------------------------------------
Set the decoder output format and initialize the decoder
-----------------------------------------------------------------*/
FORMAT G723Decoder :: SetFormat (Capabilities *pCaps)
{
	int i;

//	fOut = fopen ("test.pcm", "wb");

	m_nSamples = 0;
	for (i = 0; i < pCaps -> GetFormatCapabilityLength (); i++)
		if (pCaps -> GetFormatCapability (i) == FORMAT_PCM)
		{	
			INITdec (G723_53);
			return FORMAT_PCM;
		}
	return FORMAT_UNKNOWN;
}		

/*-----------------------------------------------------------------
Decompress a unit.
-----------------------------------------------------------------*/
BOOL G723Decoder :: Decode (
		LPBYTE pInput,
		int /*nInputLength*/,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	int nError;

	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}
	
	nError = DecSoundFrame (pInput, pOutput);
	assert (nError != DECODE_ERROR);
	rnOutputLength = GetOptimalOutputSize ();
//	fputc (3, fOut);
//	fwrite (pOutput, 1, rnOutputLength, fOut);
	return TRUE;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void G723Decoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;

	SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
	while (
		(pInData =
			(LPBYTE) GetInputStream () ->
				FetchNext (nInSize, dwTime, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		nOutSize = GetOptimalOutputSize (),
		pOutData =
			(LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
		if (pOutData == NULL)
			throw;
		if (pOutData == END_OF_STREAM)
		{	GetInputStream () -> Terminate ();
			return;
		}
		while (!Decode (pInData, nInSize, pOutData, nOutSize))
		{	pOutData =
				(LPBYTE) GetOutputStream () -> Reallocate (nOutSize);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
			{	GetInputStream () -> Terminate ();
				return;
			}
		}
		GetOutputStream () -> Dispatch (nOutSize, m_nSamples++ * 30);
		GetInputStream () -> Release (TRUE);
	}
	GetOutputStream () -> Close ();
}
