/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       Root.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of a base class for root nodes
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Root.h"
#include "Route.h"

/*-----------------------------------------------------------------
Returns a pointer to an offspring node whose nodeID = nID, or NULL 
if none exists.
-----------------------------------------------------------------*/
MediaObject * Root :: FindNode  (int nID)
{
	for (int i = m_nodes.GetLength (); --i >= 0;)
		if (m_nodes [i] -> GetID () == nID)
			return m_nodes [i];
	return NULL;
}

/*-----------------------------------------------------------------
Returns a pointer to a route routeID = nID, or NULL if none exists.
-----------------------------------------------------------------*/
Route * Root :: FindRoute  (int nID)
{
	for (int i = m_routes.GetLength (); --i >= 0;)
		if (m_routes [i] -> GetID () == nID)
			return m_routes [i];
	return NULL;
}

/*-----------------------------------------------------------------
Delete the node whose ID is nNodeID.  Returns TRUE if the node was 
found and deleted successfully.
-----------------------------------------------------------------*/
BOOL Root :: DeleteNode (int nNodeID)
{
	MediaObject *pNode = FindNode (nNodeID);
	delete pNode;
	return pNode != NULL;
}

/*-----------------------------------------------------------------
Find an offspring node nodeID is nNodeID, and replace it by a new node, 
parsed from the BIFS stream.  Returns TRUE if a node was found and 
replaced successfully.
-----------------------------------------------------------------*/
BOOL Root :: ReplaceNode (int /*nNodeID*/,  BIFSDecoder * /*pBifs*/)
{
	return FALSE;
}

/*-----------------------------------------------------------------
Delete the Route whose ID is nRouteID.  Returns TRUE if a Route was 
found and deleted successfully.
-----------------------------------------------------------------*/
BOOL Root :: DeleteRoute (int nRouteID)
{
	Route *pRoute = FindRoute (nRouteID);
	delete pRoute;
	return pRoute != NULL;
}

/*-----------------------------------------------------------------
Replace the Route whose ID is nRouteID, then that Route will be 
deleted, and a new Route with same routeID will be inserted into the 
scene.  The new route values are parsed from the BIFS stream.  Returns 
TRUE if a Route was found and replaced successfully. 
-----------------------------------------------------------------*/
BOOL Root :: ReplaceRoute (int /*nRouteID*/,  BIFSDecoder * /*pBifs*/)
{
	return FALSE;
}
