/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       Route.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of Route
-------------------------------------------------------------------------*/
#include "defs.h"
#include "MediaObj.h"
#include "NodeFld.h"
#include "Route.h"
#include "Root.h"

/*-----------------------------------------------------------------
Route constructor stores the route-id, departure node, and departure
field index, and link itself to the node field
-----------------------------------------------------------------*/
Route :: Route (int nRouteID, NodeField *pDepartureField)
{
	m_pNext = m_pNextArrival = NULL;
	m_pArrivalNode = NULL;
	m_nRouteID = nRouteID;
	m_pDepartureField = pDepartureField;
	pDepartureField -> LinkRoute (this);
}

/*-----------------------------------------------------------------
Route desctructor unlink the route from the field
-----------------------------------------------------------------*/
Route :: ~Route ()
{
	if (m_pDepartureField)
		m_pDepartureField -> UnlinkRoute (this);
	if (m_pArrivalNode)
	{	if (m_nRouteID >= 0 && m_pArrivalNode -> GetRoot () != NULL)
			m_pArrivalNode -> GetRoot () -> UnregisterRoute (this);
		m_pArrivalNode -> UnlinkRoute (this);
	}
}	

/*-----------------------------------------------------------------
Bind a Route to the arrival field, done by storing the field details
-----------------------------------------------------------------*/
void Route :: Bind (MediaObject *pArrivalNode, int nArrivalField)
{
	m_pArrivalNode = pArrivalNode;
	m_nArrivalField = nArrivalField;
	m_pArrivalNode -> LinkRoute (this);
	if (m_nRouteID >= 0 && m_pArrivalNode -> GetRoot () != NULL)
		m_pArrivalNode -> GetRoot () -> RegisterRoute (this);
}

void Route :: Activate ()
{
	if (m_pArrivalNode)
		m_pArrivalNode -> RouteEvent (
			m_pDepartureField,
			m_nArrivalField
		);
}

/*-----------------------------------------------------------------
Links itself to the associated node field
-----------------------------------------------------------------*/
void Route :: Link (Route *&rpListHead)
{
	m_pNext = rpListHead;
	rpListHead = this;
}

/*-----------------------------------------------------------------
Unlinks itself from the associated field
-----------------------------------------------------------------*/
void Route :: Unlink (Route *&rpListHead)
{
	Route *pCurrent = rpListHead;
	Route *pPrevious = NULL;

	while (pCurrent != this)
	{	if (pCurrent == NULL)
			return;
		pPrevious = pCurrent;
		pCurrent = pCurrent->m_pNext;
	}

	if (pPrevious == NULL)
		// Remove the first item in the list
		rpListHead = pCurrent->m_pNext;
	else
		pPrevious->m_pNext = pCurrent->m_pNext;
}
/*  //ML this is the old function
void Route :: Unlink (Route *&rpListHead)
{
	Route *&rpPrevious = rpListHead;
	while (rpPrevious != this)
	{	if (rpPrevious == NULL)
			return;
		rpPrevious = m_pNext;
	}
	rpPrevious = m_pNext;
}
*/

/*-----------------------------------------------------------------
Links itself to the associated arrival node
-----------------------------------------------------------------*/
void Route :: LinkArrival (Route *&rpListHead)
{
	m_pNextArrival = rpListHead;
	rpListHead = this;
}

/*-----------------------------------------------------------------
Unlinks itself from the associated arrival node
-----------------------------------------------------------------*/
void Route :: UnlinkArrival (Route *&rpListHead)
{
	Route *&rpPrevious = rpListHead;
	while (rpPrevious != this)
	{	if (rpPrevious == NULL)
			return;
		rpPrevious = m_pNextArrival;
	}
	rpPrevious = m_pNextArrival;
}
