#if !defined(_service_h)	// Sentry, use file only if it's not already included.
#define _service_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  DMIF
FILE:       Service.h
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Definition of Service
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "fmt.h"
#include "Im1defs.h"

class ALManager;
class Channel;

class Service
{
	ALManager *m_pAL;
	int m_nPendingChannels;
	ZSemaphore m_smpChannelMap;

	struct ChannelMapEntry
	{
		HANDLE m_hChannel;
		Channel *m_pChannel;
		BOOL m_bIsFlexMux;

		ChannelMapEntry (
			HANDLE hChannel = NULL,
			Channel *pChannel = NULL,
			BOOL bIsFlexMux = FALSE
		) :
			m_hChannel (hChannel),
			m_pChannel (pChannel),
			m_bIsFlexMux (bIsFlexMux)
		{}
		BOOL operator == (const ChannelMapEntry &rEntry) const
		{
			return m_hChannel == rEntry.m_hChannel;
		}
	};
	ZArray <ChannelMapEntry> m_channelMap;
	ZArray <int> m_readyChannels;

private:
	void OnChannelOpen (HANDLE hChannel, int nAssociationTag, int nES_id);
	
public:
	Service (ALManager *pAL);
	virtual ~Service ();
	ALManager *GetALManager () {return m_pAL;}
	void RequestChannels  (int nNumberOfChannels, const int *pESidList);
	virtual void Connect (LPCSTR szFileName) = 0;
	virtual void Close () = 0;
	virtual void OpenChannels (
		int nNumberOfChannels,
		const int *pESidList
	) = 0;
	virtual void ChannelsReady (
		int nNumberOfChannels,
		const int *pAssociationTagList
	) = 0;

protected:
	void OnChannelOpen (HANDLE hChannel, SMT &rStreamMapTable);
	void OnDataReceived (HANDLE hChannel, LPBYTE pData, int nPacketLength);
};

#endif                                      // _service_h sentry.
