/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       G723.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of G723Stream, the class that handles G.723
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "G723.h"
#include "assert.h"

/*-----------------------------------------------------------------
G723Stream constructor reads first character
-----------------------------------------------------------------*/
G723Stream :: G723Stream (FILE *fStream, const char* streamData) : Stream (fStream)
{
	m_cNext = getc ();
	m_lTime = 0;

	// The stream data specifies the data rate of the audio, either
	// 5.3 or 6.3 Kbits/sec.  The data rate determines the frame size.
	m_frameSize = 20;
	if ((streamData != NULL) && (streamData[0] != 0))
	{
		if (strcmp(streamData, "6.3") == 0)
			m_frameSize = 24;
		else if (strcmp(streamData, "5.3") != 0)
			fprintf(stderr, "The G723 data rate must be either 5.3 or 6.3.\n");
	}
}

/*-----------------------------------------------------------------
Time stamps are increments of 30 ms
-----------------------------------------------------------------*/
long G723Stream :: GetTimeStamp ()
{
	if (m_cNext == EOF)
		return -1;
	long lTime = m_lTime;
	m_lTime += 30;
	m_nCount = 0;
	return lTime;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the streamData field.
-----------------------------------------------------------------*/
int G723Stream :: GetChar ()
{
	if (m_nCount >= m_frameSize)
		return -1;
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

static StreamFactory<G723Stream> bifsFactory ("audio/G723");