/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       TSBifs.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of TSBifsStream, the class that handles 
			time stamped BIFS streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "TSBifs.h"

/*-----------------------------------------------------------------
Time stamped BIFS file are made of access units, each preceded by
4 bytes of time stamp and 1 byte of AU length.
-----------------------------------------------------------------*/
long TSBifsStream :: GetTimeStamp ()
{
	int c1 = getc ();
	if (c1 == EOF)
		return -1;
	int c2 = getc ();
	int c3 = getc ();
	int c4 = getc ();
	m_nAULength = getc ();
	return MAKELONG (MAKEWORD (c4, c3), MAKEWORD (c2, c1));
}

/*-----------------------------------------------------------------
GetChar returns a character and read one char ahead
-----------------------------------------------------------------*/
int TSBifsStream :: GetChar ()
{
	if (m_nAULength-- == 0)
		return -1;
	return getc ();
}

static StreamFactory<TSBifsStream> bifsFactory ("BIFS-TS");